--[[
************************************************************************
Project				: BetterGuildControl
Author				: zhinjio
Project Revision	: r13
Project Date		: 20090814172728

File				: BGC_Frames.lua
Commit Author		: zhinjio
Commit Revision		: 13
Commit Date			: 20090814172728
************************************************************************
Description	:
	Frame creation/control file

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "BetterGuildControl"
local FULLNAME = "BetterGuildControl"

local BetterGuildControl = LibStub( "AceAddon-3.0" ):GetAddon( MODNAME )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )
local LTT = LibStub:GetLibrary( "LibQTip-1.0" )

--[[
local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")
_G[ MODNAME ] = BetterGuildControl
]]--
local addon = BetterGuildControl

-- Scrolling list panel definitions...
local cell_widths = {
	["name"] = 85,
	["rank"] = 90,
	["zone"] = 150,
	["race"] = 50,
	["class"] = 75,
	["level"] = 40,
}
local cell_names = {
	[1] = "name",	[2] = "rank",	[3] = "zone",	[4] = "race",
	[5] = "class",	[6] = "level",
}
-- How many lines in the scrollframe
local NUM_LINES = 25
local PADDING = 4
local lines = {}

local list_font = CreateFont(MODNAME.."ListFont")
list_font:SetFont("Fonts\\FRIZQT__.TTF", 12)		-- Hate this font. Good enough for now.

function addon:ToggleMainPanel()
	if ( addon.MainFrame:IsVisible() ) then
		addon.MainFrame:Hide()
	else
		addon.UpdateFrames()
		addon.MainFrame:Show()
	end
end

local function ShowList(position)
	local scrollbar = mf.slider
	addon.DDebug( "list length is " .. #addon.displayList )
	if #addon.displayList <= NUM_LINES then
		scrollbar:Hide()
	else
		scrollbar:Show()
	end

	for i, line in pairs(lines) do
		addon.DDebug( "i / position" .. i .."/"..position )
		entry = addon.displayList[i + position]
		if entry then
			line.name:SetText( entry.name )
			line.rank:SetText( entry.rank )
			line.zone:SetText( entry.zone )
			line.race:SetText( entry.race )
			line.class:SetText( entry.class )
			line.level:SetText( entry.level )

			line:EnableMouse(true)
			line:SetScript("OnMouseUp", ListOnMouseUp)
			line:SetScript("OnEnter", ListOnEnter)
			line:SetScript("OnLeave", ListOnLeave)
			line:Show()
		else
			line.arg = nil
			line:EnableMouse(false)
			line:SetScript("OnMouseUp", nil)
			line:SetScript("OnEnter", nil)
			line:SetScript("OnLeave", nil)
			line:Hide()
		end
	end
end

function addon:CreateFrames()
	addon.DDebug( "Reached CreateFrames. Everything looks good." )

	-- Very generic frame creation. This will not do *any* setting of options, values
	-- or settings, since we won't know them yet at this point. There will be a subsequent
	-- cell to UpdateFrames that will be called once the db is present, etc.
	
	local fName = MODNAME .. "_MainFrame"
	addon.MainFrame = CreateFrame( "Frame", fname, UIParent )
	local mf = addon.MainFrame
	mf:SetClampedToScreen( false )
	mf:SetHeight( 400 )
	mf:SetWidth( 800 )
	mf:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 100, -100 )
	mf:EnableMouse( true )
	mf:SetMovable( true )
	mf:SetFrameLevel( 1 )
	mf:SetFrameStrata( "DIALOG" )
	mf:SetHitRectInsets( 2, 5, 4, 4 )
--	mf:Hide()
	
	mf:RegisterForDrag( "LeftButton" )
	mf:SetScript( "OnDragStart",
		function(this)
			this:StartMoving()
		end
	)
	mf:SetScript( "OnDragStop", 
		function(this)
			this:StopMovingOrSizing()
			--[[
			eventually, we'll save frame position. For now, screw it
			local db = addon.db.factionrealm[addon.me]
			db.xPos = mf:GetLeft()
			db.yPos = mf:GetTop()
			]]--
		end
	)

	local bgData = {
		bgFile = "Interface/AchievementFrame/UI-Achievement-Parchment-Horizontal-Desaturated",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false,
		tileSize = 32,
		edgeSize = 8,
		insets = { left = 2, right = 2, top = 2, bottom =2 }
	}
	mf:SetBackdrop( bgData )
	mf:SetBackdropColor( 1, 0.7, 0.5, 1 )
	mf:SetBackdropBorderColor( 1, 0.34, 0.21, 1 )

	mf:SetScript("OnMouseWheel", function(self, delta)
		local slider = self.slider
		local currentValue = slider:GetValue()
		local minValue, maxValue = slider:GetMinMaxValues()

		if delta < 0 and currentValue < maxValue then
			slider:SetValue(math.min(maxValue, currentValue + 10))
		elseif delta > 0 and currentValue > minValue then
			slider:SetValue(math.max(minValue, currentValue - 10))
		end
	end)
	
	
	
	local mf_CloseButton = CreateFrame( "Button", "mf_CloseButton", mf, "UIPanelCloseButton" )
		mf_CloseButton:SetScript( "OnClick",
			function(this)
				this:GetParent():Hide()
			end
		)
		mf_CloseButton:SetPoint( "TOPRIGHT", mf, "TOPRIGHT", 5, 5 )

	local mf_TitleText = mf:CreateFontString( "mf_TitleText", "OVERLAY", "GameFontHighlight" )
		mf_TitleText:SetText( "" )
		mf_TitleText:SetPoint( "TOP", mf, "TOP", 0, -3 )
		mf_TitleText:SetHeight( 14 )
		mf_TitleText:SetWidth( 400 )

		
-- Scrollframe stolen from GearGauge - Thanks Torhal
	addon.MainFrameList = CreateFrame( "Frame", MODNAME.."MainFrameList", mf )
	local mflist = addon.MainFrameList
	mflist:SetBackdrop( GameTooltip:GetBackdrop() )
	mflist:SetBackdropColor( 1, 1, 1, 0.5 )
	mflist:SetBackdropBorderColor( 1, 0.34, 0.21, 0.8 )
--	mflist:SetBackdropColor(GameTooltip:GetBackdropColor())
--	mflist:SetBackdropBorderColor(GameTooltip:GetBackdropBorderColor())
--	mflist:SetScale(GameTooltip:GetScale())
	mflist:SetPoint( "TOPLEFT", mf, "TOPLEFT", 20, -50 )
	mflist:SetPoint( "BOTTOMRIGHT", mf, "BOTTOMRIGHT", -200, 20 )
	mflist:SetAlpha(1)
	mflist:SetFrameStrata("DIALOG")

	-- Column headers box + frame
--	local mflist_headerbox = CreateFrame( "Frame", nil, mf )
	addon.MFListHeaderBox = CreateFrame( "Frame", nil, mf )
	local mflist_headerbox = addon.MFListHeaderBox
	mflist_headerbox:SetBackdrop( GameTooltip:GetBackdrop() )
	mflist_headerbox:SetBackdropColor( 1, 1, 1, 0.5 )
	mflist_headerbox:SetBackdropBorderColor( 1, 0.34, 0.21, 0.8 )
--	mflist_headerbox:SetScale(GameTooltip:GetScale())
	mflist_headerbox:SetAlpha(1)
	mflist_headerbox:SetHeight(25)
	mflist_headerbox:SetFrameStrata("DIALOG")
	mflist_headerbox:SetPoint( "BOTTOMLEFT", mflist, "TOPLEFT", 0, 2 )
	mflist_headerbox:SetPoint( "BOTTOMRIGHT", mflist, "TOPRIGHT", 0, 2 )
	mflist_headerbox:SetHeight( 20 )

--	local mflist_header = CreateFrame("Frame", nil, list_headerbox)
	addon.MFListHeader = CreateFrame("Frame", nil, mflist_headerbox)
	local mflist_header = addon.MFListHeader
	mflist_header:SetFrameStrata("DIALOG")
	mflist_header:SetPoint( "TOPLEFT", mflist_headerbox, "TOPLEFT", 2, -2 )
	mflist_header:SetPoint( "BOTTOMRIGHT", mflist_headerbox, "BOTTOMRIGHT", -2, -2 )
	
-------------------------------------------------------------------------------
-- List pane scrollbar.
-------------------------------------------------------------------------------
	addon.DDebug( "creating slider" )
	local scrollbar = CreateFrame("Slider", nil, mflist)
	scrollbar:SetWidth(12)
	scrollbar:SetOrientation("VERTICAL")
	scrollbar:SetBackdrop({
					bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
					edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
					tile = true, tileSize = 8, edgeSize = 8,
					insets = { left = 3, right = 3, top = 3, bottom = 3 }
			  })
	scrollbar:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Vertical")
	scrollbar:SetMinMaxValues(0, 1)
	scrollbar:SetValueStep(1)
	scrollbar:SetPoint("TOPRIGHT", mflist, "TOPRIGHT", -10, -10)
	scrollbar:SetPoint("BOTTOMRIGHT", mflist, "BOTTOMRIGHT", -10, 10)

	scrollbar:SetScript("OnValueChanged",
				function(self, value, ...)
					local offset = math.floor(value)
					ShowList(offset)
				end)
	mf.slider = scrollbar
end

function addon:UpdateFrames()
	-- This function will go through the frame values and make sure they match
	-- the current db settings, etc.
	local mf = addon.MainFrame
	mf_TitleText:SetText( FULLNAME .. " - r13" )
	local slider = mf.slider
	local currentValue = slider:GetValue()
	local offset = math.floor(value)
	ShowList(offset)
end

-------------------------------------------------------------------------------
-- Initialization functions
-------------------------------------------------------------------------------
-- Creates the lines and cells for the list panel, and normalizes the cell widths.
function addon:InitializeList()
	addon.DDebug( "listinit" )
	local mflist = addon.MainFrameList
	local mflist_header = addon.MFListHeader
	local function SetFormattedCellText(self, format, ...)
		local fs = self._string

		fs:SetJustifyH("LEFT")
		fs:SetFormattedText(tostring(format), ...)
		fs:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
		fs:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0)
		fs:Show()

		local width = fs:GetStringWidth()
		local name = self._name

		if width > cell_widths[name] then
			addon.Debug(string.format("Changing a width - %s, from %d to %d.", name, cell_widths[name], width))
			cell_widths[name] = width
		else
			width = cell_widths[name]
		end
		self:SetWidth(width)

		local height = fs:GetHeight()
		self:SetHeight(height)

		local parent = self:GetParent()

		if height > parent.height then
			parent.height = height
			parent:SetHeight(height)
		end
	end

	local function SetCellText(self, value)
		local fs = self._string

		fs:SetJustifyH("LEFT")
		fs:SetText(tostring(value))
		fs:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
		fs:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0)
		fs:Show()

		local width = fs:GetStringWidth()
		local name = self._name

		if width > cell_widths[name] then
			addon.debug(string.format("Changing a width - %s, from %d to %d.", name, cell_widths[name], width))
			cell_widths[name] = width
		else
			width = cell_widths[name]
		end
		self:SetWidth(width)

		local height = fs:GetHeight()
		self:SetHeight(height)

		local parent = self:GetParent()

		if height > parent.height then
			parent.height = height
			parent:SetHeight(height)
		end
	end

	local function CreateLineCells(line, font)
		line.height = 1

		local cell, relative

		for idx, name in ipairs(cell_names) do
			cell = CreateFrame("Frame", nil, line)
			cell:SetPoint("TOP", line)
			cell:SetPoint("BOTTOM", line)

			if relative then
				cell:SetPoint("LEFT", relative, "RIGHT", PADDING, 0)
			else
				cell:SetPoint("LEFT", line, "LEFT", 0, 0)
			end
			local fstring = cell:CreateFontString()
			fstring:SetFontObject(font)

			cell._string = fstring
			cell._name = name
			cell.SetText = SetCellText
			cell.SetFormattedText = SetFormattedCellText
			cell:Show()

			line[name] = cell
			relative = cell
		end
	end
	local anchor

	for i = 1, NUM_LINES do
		local line = CreateFrame("Frame", nil, mflist)

		if anchor then
			line:SetPoint("TOP", anchor, "BOTTOM", 0, 1)
		else
			line:SetPoint("TOP", mflist, "TOP", 0, -5)
		end
		line:SetPoint("LEFT", mflist, "LEFT", PADDING, 0)
		line:SetPoint("RIGHT", mflist, "RIGHT", -PADDING - 10, 0)

		CreateLineCells(line, list_font)
		anchor = line
		lines[i] = line
	end
	-- Initialize the list header.
	CreateLineCells(mflist_header, GameTooltipHeaderText)
	mflist_header.name:SetText( L["Name"] )
	mflist_header.rank:SetText( L["Rank"] )
	mflist_header.zone:SetText( L["Zone"] )
	mflist_header.race:SetText( L["Race"] )
	mflist_header.class:SetText( L["Class"] )
	mflist_header.level:SetText( L["Level"] )
	self.InitializeList = nil
end

--[[
************************************************************************
CHANGELOG:

08/12/09 : 
	Initial version
************************************************************************
]]--